#!/usr/bin/env python

# Copyright (c) 2017  University of Houston

import argparse
import functools
import os
import signal
import struct
import subprocess
import sys

from cachetools import LRUCache
import pcapy

INTFS = {}
POLLLIST = []
TABLESIZE = 10

BCAST = 0xffffffffffff

ETHTOOL_BIN_LIST = ["/usr/sbin/ethtool", "/sbin/ethtool"]

def parse_args ():
  parser = argparse.ArgumentParser()
  parser.add_argument("--all", dest="all", action="store_true", default=False)
  parser.add_argument("interfaces", nargs='*')
  opts = parser.parse_args()
  return opts

def find_ethtool ():
  for ethtool_bin in ETHTOOL_BIN_LIST:
    if os.path.exists(ethtool_bin):
      return ethtool_bin
  print "ERROR: ethtool binary path not found on this host - exiting"
  sys.exit(127)

def disable_offloads (intf, ethtool_bin):
  subprocess.call("%s -K %s rx off tx off sg off gro off gso off" % (ethtool_bin, intf), shell=True)

def setup(opts):
  ethtool_bin = find_ethtool()
  register_signals()
  if opts.all:
    intfs = [x for x in os.listdir("/sys/class/net") if int(open("/sys/class/net/%s/type" % (x), "r").read()) == 1]
  else: 
    intfs = opts.interfaces
  for intf in intfs:
    disable_offloads(intf, ethtool_bin)
    handle = pcapy.open_live(intf, 2000, True, 0)
    handle.setnonblock(True)
    INTFS[handle] = intf 
    POLLLIST.append((handle, functools.partial(handle_packet, handle)))

def loop():
  while True:
    for intf,handler in POLLLIST:
      intf.dispatch(1, handler)

def flood(inhandle, pkt):
  for v,h in POLLLIST:
    if v == inhandle:
      continue
    v.sendpacket(pkt)


class MacTable():

  def __init__ (self, tablesize):
    self._counter = 0
    self.mactable = {}
    self.tablesize = tablesize

  def learn (self, src, interface):
    self._counter += 1
    self.mactable[src] = (interface, self._counter)
    while len(self.mactable) > self.tablesize:
      minmac = None
      for k,v in self.mactable.iteritems():
        if minmac:
          if v[1] < minmac[0]:
            minmac = (v[1], k)
        else:
          minmac = (v[1], k)
      del self.mactable[minmac[1]]

  def dump (self):
    return self.mactable

  def lookup (self, dst):
    try:
      return self.mactable[dst][0]
    except KeyError:
      return None

  def clear (self):
    self._counter = 0
    self.mactable = {}

MACTABLE = MacTable(TABLESIZE)

def handle_packet (intf_handle, record_header, data):
  # intel ok... don't use in arm or ppc
  (d1,d2,s1,s2) = struct.unpack("!HIHI", data[:12])
  src = s2 | (s1<<32)
  dst = d2 | (d1<<32)
  MACTABLE.learn(src, intf_handle)

  if dst == BCAST:
    flood(intf_handle, data)
    return

  out = MACTABLE.lookup(dst)

  if out:
    out.sendpacket(data)
  else:
    flood(intf_handle, data)

def dump_table (num, frame):
  f = open("/L2Table.txt", 'w')
  table = MACTABLE.dump()
  for k,v in table.iteritems():
    f.write("%d: %s\n" % (k, INTFS[v[0]]))
    # print "%d: %s" % (k, INTFS[v[0]])
  f.flush()

def clear_table (num, frame):
  MACTABLE.clear()

def register_signals ():
  signal.signal(signal.SIGUSR1, dump_table)
  signal.signal(signal.SIGUSR2, clear_table)

if __name__ == '__main__':
  opts = parse_args()
  setup(opts)
  loop()
