# Copyright (c) 2017  University of Houston

from __future__ import absolute_import

from .bases import AlpineSupervisorHost, _wrapPOA, _wrapMapPOA
from .util import RSYSLOGCONF

from foam.vts.drivers.docker import DockerDriver, HostPathVolume
from foam.vts.hostdriver import HostDriver
from foam.vts.data import UnknownInstanceError

STORE = "%s/uh.bind/%s/%s"

NAMEDCONF = """//%(tld)s
options {
        directory "/data";
        listen-on { %(ipAddress)s; 127.0.0.1;};
        listen-on-v6 { none; };             
        allow-transfer { none; };
        allow-recursion { any; };
        allow-query { any; };
        allow-query-cache { any; };
};                                       
zone "%(tld)s" IN {
        type master;   
        file "db.tld";
};                                   
zone "." {
        type master;
        file "db.root";
};
zone "localhost" IN{
        type master;
        file "db.localhost";
};
"""
TOPZONEFILE = """$ORIGIN %(tld)s.
@       1D      IN      SOA     %(vmName)s.%(tld)s. hostmaster.%(tld)s. ( 1 3h 15 1w 3h)
                IN      NS      %(vmName)s.%(tld)s.
%(vmName)s              IN      A       %(ipAddress)s
%(delegationRecords)s

"""
ROOTZONEFILE = """$TTL    86400 ;
$ORIGIN .
.               IN      SOA     %(vmName)s.%(tld)s. hostmaster.%(tld)s. ( 1 3h 15 1w 3h )
                IN      NS      %(vmName)s.%(tld)s.
%(tld)s     IN      NS      %(vmName)s.%(tld)s.
%(vmName)s.%(tld)s  IN      A       %(ipAddress)s
"""
LOCALHOSTZONE = """$TTL	86400 ;
$ORIGIN localhost.
@       1D      IN      SOA     %(vmName)s.%(tld)s. hostmaster.%(tld)s. ( 1 3h 15 1w 3h)
@       1D      IN      NS      @ 
        1D      IN      A       127.0.0.1
"""


class BindHandler(AlpineSupervisorHost):
  def prebuild (self, cobj):
    '''write configs, mount in /data'''
    self.log.debug("Prebuild (%s, %s)" % (cobj._sliver.uuid, cobj.client_id))
    hd = HostDriver(self.log.info)
    dpath = STORE % (hd.volroot, cobj._sliver.uuid, cobj.client_id)
    os.makedirs(dpath)
    self.volumes.append(HostPathVolume(dpath, "/data"))

  @staticmethod
  def geniSetup (registry):
    AlpineSupervisorHost.geniSetup(registry)
