# Copyright (c) 2017  University of Houston

from __future__ import absolute_import

from .bases import AlpineSupervisorHost, _wrapPOA, _wrapMapPOA

from foam.vts.drivers.docker import DockerDriver, HostPathVolume
from foam.vts.hostdriver import HostDriver

CONFIG = """
#user www;
worker_processes %(workers)d;
pid /cfg/nginx.pid;

events {
  worker_connections 128;
}

http {
  access_log syslog:server=%(syslog-ip)s,facility=local7,tag=nginx,severity=info combined;
  error_log syslog:server=%(syslog-ip)s debug;
  gzip on;

  include /etc/nginx/mime.types;
  default_type application/octet-stream;

  server {
    listen %(port)d;

    root /www-root;
    sendfile  on;
    output_buffers  1 512;
    read_ahead  512K;

    location / {
      autoindex on;
      proxy_max_temp_file_size 0;
    }
  }
}
"""

class NginxHandler(AlpineSupervisorHost):
  def _writeConfig (self, hpath):
    data = {"workers" : self.attrs["worker-processes"], "port" : self.attrs["listen-port"],
            "syslog-ip" : str(self.attrs["syslog-ip"])}
    with open("%s/nginx.conf" % (hpath), "w+") as cfgf:
      cfgf.write(CONFIG % (data))

  def postlocaltopo (self, cobj):
    hvol = None
    for vol in self.container._vols:
      if vol.cpath == "/cfg":
        hvol = vol
        break

    self._writeConfig(hvol.hpath)

  def postnettopo (self, cobj):
    dd = DockerDriver(self.log.info)
    dd.ccall(self.container, "/usr/bin/ssh-keygen -A")
    dd.supervisor(self.container, "start", "nginx")
    dd.supervisor(self.container, "start", "sshd")

  @staticmethod
  def geniSetup (registry):
    AlpineSupervisorHost.geniSetup(registry)
