# Copyright (c) 2016-2017  University of Houston

from __future__ import absolute_import

import os
import os.path
import sys

from .bases import AlpineHost, SupervisorHost, _wrapPOA
from foam.vts.drivers.docker import DockerDriver
from foam.types import EthernetMAC

class VSwitchHandler(AlpineHost, SupervisorHost):

  def postlocaltopo (self, cobj):
    dd = DockerDriver(self.log.debug)
    (o,e,r) = dd.supervisor(self.container, "start", "vswitch")

  def getL2Table (self, options, user_urn):

    retl2t = []

    dd = DockerDriver(self.log.debug)

    (o,e,r) = dd.supervisor(self.container, "pid", "vswitch")
    pid = o.strip()

    (o,e,r) = dd.ccall(self.container, "kill -SIGUSR1 %s" % (pid))

    (o,e,r) = dd.ccall(self.container, "cat /L2Table.txt")
    
    entries = o.strip().split("\n")
    
    retl2t.append(("Port", "VLAN", "MAC", "Age"))
    if len(entries) > 0:
      for item in entries:
        if len(item) > 1:
          mac, intf = item.split(":")
          pretty_mac = str(EthernetMAC(int(mac)))
          d = (intf.strip(), 0, pretty_mac, None)
          retl2t.append(d)
    return retl2t


  def clearL2Table (self, options, user_urn):

    retl2t = []

    dd = DockerDriver(self.log.debug)

    (o,e,r) = dd.supervisor(self.container, "pid", "vswitch")
    pid = o.strip()

    (o,e,r) = dd.ccall(self.container, "kill -SIGUSR2 %s" % (pid))
    retl2t.append(("Port", "VLAN", "MAC", "Age"))

    return retl2t
 
  @staticmethod
  def geniSetup (registry):
    SupervisorHost.geniSetup(registry)
    AlpineHost.geniSetup(registry)

    registry.addPOACallback(*(VSwitchHandler.poaGetL2Table))
    registry.addPOACallback(*(VSwitchHandler.poaClearL2Table))

VSwitchHandler.poaGetL2Table = _wrapPOA("api:l2-switch:get-l2-table", VSwitchHandler, "getL2Table")
VSwitchHandler.poaClearL2Table = _wrapPOA("api:uh.vswitch:clear-l2-table", VSwitchHandler, "clearL2Table")
